(* ::Package:: *)

(* :Context: Graphics`Fractals` *)

(* :Summary:
   Density and contour representation of Julia and Mandelbrot sets,generalized Julia and
   mandelbrot sets and the attraction basins of roots of z^p-1 ecuation obtained by the 
   Newton-Raphson method for p arbitrary complex value.
 *)

(* :Copyright: (C) 2007 by Francisco Javier Sanchez Chapado *)

(* email of the autor: fjschapado@gmail.com *)

(* :Package Version: 1.2 *)

(* :Mathematica Version: 5.2 *)

(* :History:
   Version 1.2 is an improvement of the version 1.0.
   New in this version:
       - Added functions for contour plots of all fractals.
       - The order in the fractals now can be an arbitrary complex number. 
       - The specification of parameter Const in JuliaPlot now are made in a more natural
         way with a complex value not with a two elements list like in 1.0 version.
       - This version is, in general, faster than 1.0 version.
       - The algoritms are programmed in a more simple and clear way.
      
      
*)

(* :Keywords: 
   Julia, Mandelbrot, Mandel, Newton, Newton-Raphson 
*)

(* :Sources:   
*)

(* :Discussion:
*)

(* :Requirements:
   Utilities`FilterOptions`
*)

BeginPackage["Graphics`Fractals`"]


JuliaPlot::usage="MandelbrotPlot[{x,x0,x1},{y,y0,y1}] Represent the density plot of Julia 
      set in the specificated zone.";

MandelbrotPlot::usage="MandelbrotPlot[{x,x0,x1},{y,y0,y1}] Represent the density plot of 
      Mandelbrot set in the specificated zone.";

NewtonRootsPlot::usage="NewtonRootsPlot[{x,x0,x1},{y,y0,y1}] Represent the density plot of
      attraction basins of roots of z^p-1 ecuation obtained by the Newton-Raphson method.";
      
JuliaContourPlot::usage="MandelbrotContourPlot[{x,x0,x1},{y,y0,y1}] Represent the contour 
      plot of Julia set in the specificated zone.";

MandelbrotContourPlot::usage="MandelbrotContourPlot[{x,x0,x1},{y,y0,y1}] Represent the 
      contour plot of Mandelbrot set in the specificated zone.";

NewtonRootsContourPlot::usage="NewtonRootsContourPlot[{x,x0,x1},{y,y0,y1}] Represent the 
      contour plot of attraction basins of roots of z^p-1 ecuation obtained by the 
      Newton-Raphson method.";

Maxiter::usage = "Maxiter->{n} Maximum number of iterations."

Order::usage = "Order->{n} Order of fractal." 

Const::usage = "Const-> specifies the complex constant used in the Julia set."


Begin["`Private`"]

Needs["Utilities`FilterOptions`"]

Options[JuliaPlot] = { Maxiter->100, Order->2, Const->0.3+0.6*I }
Options[MandelbrotPlot] = { Maxiter->35, Order->2 }
Options[NewtonRootsPlot] = { Maxiter->125, Order->3 }
Options[JuliaContourPlot] = { Maxiter->100, Order->2, Const->0.3+0.6*I }
Options[MandelbrotContourPlot] = { Maxiter->35, Order->2 }
Options[NewtonRootsContourPlot] = { Maxiter->125, Order->3 }



(* ALGORITHMS TO CALCULATE THE FRACTALS *)

(*The Julia set*)
Julia=Compile[{zreal, zimag, ci, expreal, expimag, constreal, constimag},
    Module[{z=zreal+zimag*I, niter=0, imax=ci, n=expreal+expimag*I, z0=constreal+constimag*I},
        While[ Abs[z]<=100,                             
            z=z^n+z0;                               
            niter++;
            If[niter>imax,Break[]]
        ];
        niter
    ]
];

(*The Mandelbrot set*)
Mandelbrot=Compile[{zreal,zimag,ci,expreal,expimag},
    Module[{z=zreal+zimag*I,niter=0,z0,imax=ci,n=expreal+expimag*I},                                                
        z0=z;
        While[Abs[z]<100,
            z=z^n+z0;
            niter++;
            If[niter>imax,Break[]]
        ];
        niter
    ]
];

(*Attraction basins of roots of z^n-1 ecuation obtained by the Newton-Raphson method*)
NewtonRoots=Compile[{zreal,zimag,ci,expreal,expimag},
    Module[{z=zreal+zimag*I,zold = z,niter=0,imax=ci,n=expreal+expimag*I},
        While[niter<=imax,
            zold = z;
            (*z = ((n-1)*z^n+1)/(n*z^(n-1));*)
            (* The next version of above formula is faster: *)
            z = z*((n-1)+z^(-n))/n;                             
            If[Abs[z^n-zold^n]<0.00001,Break[]];
            niter++
        ];
        niter
    ]   
];



(* DENSITY REPRESENTATION FUNCTIONS *)

(* Density plot of the Julia set *)
JuliaPlot[{xp_, ap_, bp_},{yp_, cp_, dp_}, opts___]:=
    Module[{i,o,z0},
        i = Maxiter /. {opts} /. Options[JuliaPlot][[1]] ;       
        o = Order /. {opts} /. Options[JuliaPlot][[2]] ; 
        z0 = Const /. {opts} /. Options[JuliaPlot][[3]] ;
        DensityPlot[Julia[xp,yp,i,Re[o],Im[o],Re[z0],Im[z0]],{xp,ap,bp},{yp,cp,dp},
        Evaluate[FilterOptions[DensityPlot,opts]],PlotPoints->200,
        Mesh->False,AspectRatio->Automatic
    ]
];

(* Density plot of the Mandelbrot set *)
MandelbrotPlot[{xp_,ap_,bp_},{yp_,cp_,dp_},opts___]:=
    Module[{i,o},
        i = Maxiter /. {opts} /. Options[MandelbrotPlot][[1]] ;  
        o = Order /. {opts} /. Options[MandelbrotPlot][[2]] ;    
        DensityPlot[Mandelbrot[xp,yp,i,Re[o],Im[o]],{xp,ap,bp},{yp,cp,dp},
        Evaluate[FilterOptions[DensityPlot,opts]],PlotPoints->200,
        Mesh->False,AspectRatio->Automatic
    ]
];

(* Density plot of attraction basins of roots of z^p-1 ecuation obtained by the Newton-Raphson method. *)
NewtonRootsPlot[{xp_,ap_,bp_},{yp_,cp_,dp_},opts___]:=
    Module[{i},
        i = Maxiter /. {opts} /. Options[NewtonRootsPlot][[1]] ; 
        o = Order /. {opts} /. Options[NewtonRootsPlot][[2]] ;   
        DensityPlot[NewtonRoots[xp,yp,i,N[Re[o]],N[Im[o]]],{xp,ap,bp},{yp,cp,dp},
        Evaluate[FilterOptions[DensityPlot,opts]],PlotPoints->200,
        Mesh->False,AspectRatio->Automatic
    ]
];
                                                    
                                                    
                                
(* CONTOUR REPRESENTATION FUNCTIONS *)

(* Contour plot of the Julia set *)
JuliaContourPlot[{xp_, ap_, bp_},{yp_, cp_, dp_}, opts___]:=
    Module[{i,o,z0},
        i = Maxiter /. {opts} /. Options[JuliaPlot][[1]] ;       
        o = Order /. {opts} /. Options[JuliaPlot][[2]] ; 
        z0 = Const /. {opts} /. Options[JuliaPlot][[3]] ;
        ContourPlot[Julia[xp,yp,i,Re[o],Im[o],Re[z0],Im[z0]],{xp,ap,bp},{yp,cp,dp},
        Evaluate[FilterOptions[ContourPlot,opts]],ContourShading->False,AspectRatio->Automatic
    ]
];

(* Contour plot of the Mandelbrot set *)
MandelbrotContourPlot[{xp_,ap_,bp_},{yp_,cp_,dp_},opts___]:=
    Module[{i,o},
        i = Maxiter /. {opts} /. Options[MandelbrotPlot][[1]] ;  
        o = Order /. {opts} /. Options[MandelbrotPlot][[2]] ;    
        ContourPlot[Mandelbrot[xp,yp,i,Re[o],Im[o]],{xp,ap,bp},{yp,cp,dp},
        Evaluate[FilterOptions[ContourPlot,opts]],ContourShading->False,AspectRatio->Automatic
    ]
];

(* Contour plot of attraction basins of roots of z^p-1 ecuation obtained by the Newton-Raphson method. *)
NewtonRootsContourPlot[{xp_,ap_,bp_},{yp_,cp_,dp_},opts___]:=
    Module[{i},
        i = Maxiter /. {opts} /. Options[NewtonRootsPlot][[1]] ; 
        o = Order /. {opts} /. Options[NewtonRootsPlot][[2]] ;   
        ContourPlot[NewtonRoots[xp,yp,i,N[Re[o]],N[Im[o]]],{xp,ap,bp},{yp,cp,dp},
        Evaluate[FilterOptions[ContourPlot,opts]],ContourShading->False,AspectRatio->Automatic
    ]
];



End[]

Protect[Mandelbrot,Julia,NewtonRoots,JuliaPlot,MandelbrotPlot,NewtonRootsPlot,JuliaContourPlot,
        MandelbrotContourPlot,NewtonRootsContourPlot]

EndPackage[]
